/*---------------------------------------------------------------*/
/* 8-pin SD audio player R0.04                     (C)ChaN, 2010 */
/*---------------------------------------------------------------*/
#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include "diskio.h"
#include "pff.h"

#define FCC(c1,c2,c3,c4)	(((DWORD)c4<<24)+((DWORD)c3<<16)+((WORD)c2<<8)+(BYTE)c1)	/* FourCC */

// Define LED grid ports
// Each of the grid ports are connected to two rows of leds.
// The upper 4 bits is one row, the lower 4 bits are one row.
#define GRID1 PORTC
#define GRID2 PORTA
#define LAYER_PORT PORTD

// *** Cube buffer ***
// The 3D image displayed on the cube is buffered in a 2d array 'cube'.
// The 1st dimension in this array is the Z axis of the cube.
// The 2nd dimension of the array is the Y axis.
// Each byte is a stripe of leds running along the X axis at the given
// Z and Y coordinates.
// Only the 4 lower bits are used, since the cube is only 4x4x4.
// This buffer design was chosen to have code compatability with a 8x8x8 cube.
// "volatile" makes the variables reachable from within the interrupt functions
volatile unsigned char cube[4][4];

// What layer the interrupt routine is currently showing.
volatile unsigned char current_layer;


// Fill the cube with a given pattern.
// fill(0x00); clears the cube
// fill(0xff); lights all leds
void fill (unsigned char pattern)
{
	int z;
	int y;
	for (z=0;z<4;z++)
	{
		for (y=0;y<4;y++)
		{
			cube[z][y] = pattern;
		}
	}
}


/*---------------------------------------------------------*/
/* Work Area                                               */
/*---------------------------------------------------------*/

volatile BYTE FifoRi, FifoWi, FifoCt;	/* FIFO controls */

unsigned char Buff1[256];		/* Wave output FIFO */
unsigned char Buff2[256];		/* Wave output FIFO */

FATFS Fs;			/* File system object */
DIR Dir;			/* Directory object */
FILINFO Fno;		/* File information */

WORD rb;			/* Return value. Put this here to avoid bugs of avr-gcc */


/*---------------------------------------------------------*/

static
DWORD load_header (unsigned char *presc)	/* 0:Invalid format, 1:I/O error, >1:Number of samples */
{
	DWORD sz;


	if (pf_read(Buff1, 12, &rb)) return 1;	/* Load file header (12 bytes) */

	if (rb != 12 || LD_DWORD(Buff1+8) != FCC('W','A','V','E')) return 0;

	for (;;) {
		pf_read(Buff1, 8, &rb);			/* Get Chunk ID and size */
		if (rb != 8) return 0;
		sz = LD_DWORD(&Buff1[4]);		/* Chunk size */

		switch (LD_DWORD(&Buff1[0])) {	/* FCC */

		case FCC('f','m','t',' ') :					/* 'fmt ' chunk */
			if (sz > 100 || sz < 16) return 0;		/* Check chunk size */
			pf_read(Buff1, sz, &rb);					/* Get content */
			if (rb != sz) return 0;
			if (Buff1[0] != 1) return 0;				/* Check coding type (1) */
			if (Buff1[2] != 1 && Buff1[2] != 2) 		/* Check channels (1/2) */
				return 0;
			
			if (Buff1[14] != 8 && Buff1[14] != 16)	/* Check resolution (8/16) */
				return 0;
			
			//OCR0 = (BYTE)(F_CPU/8/LD_WORD(&Buff1[4]))-1;	/* Sampling freq */
            if (F_CPU/8/LD_WORD(&Buff1[4]) < 256){
                *presc = 8;
                OCR0 = (BYTE)(F_CPU/8/LD_WORD(&Buff1[4]))-1;    /* Sampling freq */
            }
            else {
                *presc = 64;
                OCR0 = (BYTE)(F_CPU/64/LD_WORD(&Buff1[4]))-1;
              }
			
			break;

		case FCC('d','a','t','a') :				/* 'data' chunk (start to play) */
			return sz;

		case FCC('L','I','S','T') :				/* 'LIST' chunk (skip) */
		case FCC('f','a','c','t') :				/* 'fact' chunk (skip) */
			pf_lseek(Fs.fptr + sz);
			break;

		default :								/* Unknown chunk (error) */
			return 0;
		}
	}

	return 0;
}




volatile uint8_t read1 = 1;

volatile uint16_t counter=0;

volatile uint8_t sync = 0;

volatile uint16_t vu=0;

ISR(TIMER0_COMP_vect)
{

	if(sync == 1)
	{
		
		if(read1 == 1)
		{
			OCR1A = (unsigned char)Buff1[counter++];
			
			if(counter == 256)
			{
				read1 = 0;
				counter = 0;
			}
		}
		else if(read1 == 0)
		{
			OCR1A = (unsigned char)Buff2[counter++];
			if(counter == 256)
			{
				read1 = 1;
				counter = 0;
			}
		}
	}
    vu++;

	// Take the current 2D image at the current layer along the Z axis
	// and place it on the LED grid.
	GRID1 = (0x0f & cube[current_layer][0]) | (0xf0 & (cube[current_layer][1] << 4));
	GRID2 = (0x0f & cube[current_layer][2]) | (0xf0 & (cube[current_layer][3] << 4));

    
    if(vu == 650)

    {
        PORTC = 0x00;

        if(OCR1A < 100)
            LAYER_PORT = (0x01 << 3);

        else if(OCR1A < 150)
            LAYER_PORT = (0x01);

        else if(OCR1A < 200)
            LAYER_PORT = (0x01 << 1);

        else if(OCR1A < 256)
            LAYER_PORT = (0x01 << 2);

        vu = 0;
    }
    
}


void Timer0_init(uint8_t div)
{
    TCCR0 = _BV(WGM01);

    switch(div){
        case 8:
            TCCR0 |= (1<<CS01);
            break;
        case 64:
            TCCR0 |= (1<<CS01) | (1<<CS00);
            break;
        default:
            TCCR0 |= (1<<CS00);
            break;
        }

    TIMSK = _BV(OCIE0);
}

void Timer1_init(void)
{

	TCCR1A = _BV(COM1A1)|_BV(COM1A0)|_BV(WGM10);
	TCCR1B = _BV(WGM12)|_BV(CS10);
}


static UINT play (const char *fn)
{
	DWORD sz;
	FRESULT res;
	unsigned char read2 = 8;

	
	uint16_t size = 256;
	

	if ((res = pf_open(fn)) == FR_OK) {
	
		
		sz = load_header(&read2);			/* Load file header */
		if (sz < 256) return (UINT)sz;

		Timer0_init(read2);
		Timer1_init();

        sei();
		
		
		FifoCt = 0; FifoRi = 0; FifoWi = 0;		/* Reset FIFO */
		pf_read(Buff1, 512 - (Fs.fptr % 512), &rb);	/* Snip sector unaligned part */
		sz -= rb;

			
		res = pf_read(Buff1, size, &rb);
		if (res != FR_OK) { while (FifoCt) ; cli(); return res; }
		
		sync = 1;
				
		do {
				
			read2 = read1;
			
			if(read1 == 1)	
				res = pf_read(Buff2, size, &rb);
			else
				res = pf_read(Buff1, size, &rb);
				
			if (res != FR_OK) { PORTC = 0x08; break; }
			
			while(read2 == read1);
			
			if(!(PINB&(1<<PB1))) 
			{
				sync = 0;
				_delay_ms(500); 
				break;
			}
			
		
		} while (rb == size);	/* Repeat until all data read */
		
		sync = 0;
	}

	while (FifoCt) ;			/* Wait for FIFO empty */
    cli();
	
	return res;
}


/*-----------------------------------------------------------------------*/
/* Main                                                                  */


int main (void)
{
	

    DDRD |= 0xff;  
    DDRA |= 0xff;  
    DDRC |= 0xff;
	DDRB &= ~(1<<PB1); //intrare

	PORTB |= (1<<PB1);

    PORTD &= ~((1<<PD5)|(1<<PD6) |(1<<PD7));
    current_layer = 3;
	fill(0xff);
	

	for (;;) {
        
		_delay_ms(1000);					/* Delay 1000ms */

		if (pf_mount(&Fs)) 
		{	
            
			continue;	// Initialize FS 
		}
        PORTD |= (1<<PD7);

		for (;;) {
			if (pf_opendir(&Dir, "")) break;	// Open sound file directory (root dir) 
			while (!pf_readdir(&Dir, &Fno) && Fno.fname[0]) {	// Play all wav files in the dir //
				if (!(Fno.fattrib & (AM_DIR|AM_HID)) && strstr(Fno.fname, ".WAV")) {
					if (play(Fno.fname)) break;
				}
			}
		}
	}
}



